// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bri++.H"


//  Define this to print some debugging information
//
//#define DEBUG_LIST


intervalList::intervalList() {
  _isSorted = true;
  _listLen  = 0;
  _listMax  = 16;
  _list     = new _intervalPair [_listMax];
}


intervalList::~intervalList() {
  delete [] _list;
}


intervalList &
intervalList::operator=(intervalList &src) {
  _isSorted = src._isSorted;
  _listLen = src._listLen;

  if (_listMax < src._listMax) {
    delete [] _list;
    _listMax = src._listMax;
    _list    = new _intervalPair [_listMax];
  }

  memcpy(_list, src._list, _listLen * sizeof(_intervalPair));

  return(*this);
}

void
intervalList::add(u32bit position, u32bit length) {

#ifdef DEBUG_LIST
  fprintf(stderr, "Adding %u - %u\n", position, position+length);
#endif

  if (_listLen >= _listMax) {
    _listMax *= 2;
    _intervalPair *l = new _intervalPair [_listMax];
    memcpy(l, _list, sizeof(_intervalPair) * _listLen);
    delete [] _list;
    _list = l;
  }

  _list[_listLen].lo = position;
  _list[_listLen].hi = position + length;

  if ((_listLen > 0) &&
      (_list[_listLen-1].lo > _list[_listLen].lo)) {
#ifdef DEBUG_LIST
    fprintf(stderr, "list isn't sorted\n");
#endif
    _isSorted = false;
  }

  _listLen++;
}


static
int
intervalList_sort_helper(const void *a, const void *b) {
  _intervalPair *A = (_intervalPair *)a;
  _intervalPair *B = (_intervalPair *)b;

  if (A->lo < B->lo)
    return(-1);
  if (A->lo > B->lo)
    return(1);
  return(0);
}


void
intervalList::sort(void) {

  if (_isSorted) {
#ifdef DEBUG_LIST
    fprintf(stderr, "List is already sorted!\n");
#endif
    return;
  }

  if (_listLen > 1)
    qsort(_list, _listLen, sizeof(_intervalPair), intervalList_sort_helper);

  _isSorted = true;
}


void
intervalList::merge(void) {
  u32bit  targetInterval  = 0;
  u32bit  examineInterval = 1;

  if (_listLen < 2)
    return;

  sort();

  while (examineInterval < _listLen) {

    if ((_list[targetInterval].lo == 0) &&
        (_list[targetInterval].hi == 0)) {

      //  Our target interval is empty.  Copy in the interval we are
      //  examining and move to the next.

      _list[targetInterval].lo = _list[examineInterval].lo;
      _list[targetInterval].hi = _list[examineInterval].hi;
      examineInterval++;
    } else {

      //  We have a valid targetInterval.  See if it overlaps
      //  with the interval we are examining.

      if (_list[examineInterval].lo <= _list[targetInterval].hi) {

        //  Got an intersection.

        //  Merge examineInterval into targetInterval -- the hi range
        //  is extended if the examineInterval range is larger.
        //
        if (_list[targetInterval].hi < _list[examineInterval].hi)
          _list[targetInterval].hi = _list[examineInterval].hi;
        
        //  Clear the just merged examineInterval
        //
        _list[examineInterval].lo = 0;
        _list[examineInterval].hi = 0;

        //  Move to the next interval to examine.
        //
        examineInterval++;
      } else {

        //  No intersection.  Move along.  Nothing to see here.

        //  If there is a gap between the target and the examine (we already
        //  merged sometime in the past), copy examine to the next target.

        targetInterval++;

        if (targetInterval != examineInterval) {
          _list[targetInterval].lo = _list[examineInterval].lo;
          _list[targetInterval].hi = _list[examineInterval].hi;
        }

        examineInterval++;
      }
    }
  }

  if (targetInterval+1 < _listLen)
    _listLen = targetInterval + 1;
}

