// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef READ_BUFFER_H
#define READ_BUFFER_H

class readBuffer {
public:
  //  If bufferMax is zero, then the file is accessed using memory
  //  mapped I/O.  Otherwise, a small buffer is used.
  //
  readBuffer(const char *filename, u32bit bufferMax = 32 * 1024);

  //  This constructor always uses a small buffer; memory mapped I/O
  //  is not allowed.  This lets us pass in fileno(stdin).
  //
  readBuffer(int fileptr, const char *filename, u32bit bufferMax = 32 * 1024);

  ~readBuffer();

  bool      eof(void);
  bool      next(void);
  char      get(void);
  char      getnext(void);
  bool      seek(off_t pos);
  size_t    read(char *buf, size_t len);   //  read the next len bytes into the user buffer buf
  off_t     tell(void);

private:
  void      fillBuffer(void);
  void      init(int fileptr, const char *filename, u32bit bufferMax);

  char    *_filename;

  int      _file;
  int      _fileType;  //  0=stdin, 1=normal, 2=mmap
#ifdef __APPLE__
  int      _pad1;
#endif
  off_t    _filePos;

  bool     _eof;

  //  If bufferMax is zero, then we are using the mmapped interface, otherwise,
  //  we are using a open()/read() and a small buffer.

  size_t   _bufferPos;
  size_t   _bufferLen;
  size_t   _bufferMax;
  char    *_buffer;
#ifdef __APPLE__
  int      _pad2;
#endif
};


inline
bool
readBuffer::eof(void) {
  return(_eof);
}


inline
bool
readBuffer::next(void) {

  if (_eof)
    return(true);

  switch (_fileType) {
    case 0:
    case 1:
      _bufferPos++;
      _filePos++;

      if (_bufferPos >= _bufferLen)
        fillBuffer();
      break;
    case 2:
      _bufferPos++;
      if (_bufferPos >= _bufferLen)
        _eof = true;
      break;
  }

  return(_eof);
}


inline
char
readBuffer::get(void) {
  return(_buffer[_bufferPos]);
}


inline
char
readBuffer::getnext(void) {
  char  x = _buffer[_bufferPos];
  next();
  return(x);
}


inline
off_t
readBuffer::tell(void) {

  if (_fileType == 2)
    return((size_t)_bufferPos);

  return(_filePos);
}


#endif  //  READ_BUFFER_H
