#!/usr/bin/perl

# This file is part of ESTmapper.
# Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
# Authors: Brian Walenz and Liliana Florea
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received (LICENSE.txt) a copy of the GNU General Public 
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



use strict;

#  Not in ESTmapper install directory.  Bummer.
use lib "/IR/devel/mapping/cds/IR/BRI/scripts";
use sim4polish;
use fasta;

my $leaff    = "/usr/local/ir/bin/ESTmapper/leaff";
my $filter   = "/usr/local/ir/bin/ESTmapper/filterPolishes";


#
#  Utility to search an ESTmapper run directory and find exactly what
#  happened to a sequence.  The sequence is specified as an
#    ATAC index
#    ESTmapper internal index
#  They're both the same thing.  It's the sequence ordinal, starting
#  at zero.
#

if (scalar(@ARGV != 2)) {
    print STDERR "usage: $0 <path-to-estmapper-run-directory> <id>\n";
    exit(1);
}

my $path = shift @ARGV;
my $id   = shift @ARGV;


#  We should check that the directory exists, and that it's for a completed
#  run of estmapper.


#  First, grab the defline from the sequence file.  This also lets us
#  confirm the sequence with the user and make sure it really does
#  exist in the input.
#
system("$leaff -F $path/0-input/cDNA.fasta -s $id | head -1") and die "leaff failed.\n";


#  Read the good matches, save
#
my @g;
open(F, "< $path/polishes-good") or die "Failed to open $path/polishes-good\n";
while (!eof(F)) {
    my %p = &sim4polish::readPolish(*F);
    if ($p{'estID'} eq $id) {
        push @g, "identity=$p{'percentID'} coverage=$p{'coverage'} genomic=$p{'dbID'}\0$p{'raw'}";
    }
}
close(F);


#  Read all the raw polishes
#
my @r;
open(F, "cat $path/3-polish/*polished |") or die "Failed to open raw polishes\n";
while (!eof(F)) {
    my %p = sim4polish::readPolish(*F);
    if ($p{'estID'} eq $id) {
        push @r, "identity=$p{'percentID'} coverage=$p{'coverage'} genomic=$p{'dbID'}\0$p{'raw'}";
    }
}
close(F);


print STDERR "Found ", scalar(@g), " polishes in polishes-good.\n";
foreach my $m (@g) {
    my ($a, $b) = split '\0', $m;
    print STDERR "  $a\n";
}
print STDERR "Found ", scalar(@r), " polishes in all raw polishes.\n";
foreach my $m (@r) {
    my ($a, $b) = split '\0', $m;
    print STDERR "  $a\n";
}
