#!/usr/bin/perl

# This file is part of ESTmapper.
# Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
# Authors: Brian Walenz and Liliana Florea
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received (LICENSE.txt) a copy of the GNU General Public 
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



#  usage: waitfor.pl <usename> [mincpu] [waittime] [sleeptime]
#
#  monitors the machine for jobs running by <username>.  If no jobs
#  are found for a fixed time window, the script exits.
#
#  The name of the user to watch is the first argument
#
#  If there is a second argument, it's the amount of CPU the
#  user can be using and still be considered idle.
#
#  If there is a third argument, it's the grace period before we
#  exit.
#
#  If there is a forth argument, it's the time to sleep between
#  checking.
#
#
#
my $userToWatch = shift @ARGV;

my $idleCPU = 10;
if (scalar(@ARGV) > 0) {
    $idleCPU = shift @ARGV;
}

my $timeToWait = 5 * 60;
if (scalar(@ARGV) > 0) {
    $timeToWait = shift @ARGV;
}

my $sleepTime = 10;
if (scalar(@ARGV) > 0) {
    $sleepTime = shift @ARGV;
}

my $stillRunning = $timeToWait;

while ($stillRunning > 0) {
    sleep($sleepTime);

    my $cpuInUse  = 0;

    open(F, "ps aux |");
    while (<F>) {
        #  USER PID CPU MEM VSZ RSS
        if (m/\s*(\w+)\s+(\d+)\s+(\d+\.\d+)\s+(\d+\.\d+)\s+(\S+)\s+(\S+)\s+/) {
            my $usr = $1;
            my $pid = $2;
            my $cpu = $3;
            my $mem = $4;
            my $vsz = $5;
            my $rss = $6;

            if ($usr eq $userToWatch) {
                $cpuInUse += $cpu;
            }
        }
    }
    close(F);

    if ($cpuInUse >= $idleCPU) {
        $stillRunning = $timeToWait;
    } else {
        $stillRunning -= $sleepTime;
    }

    #print "$userToWatch\tcpu=$cpuInUse\ttimeLeft=$stillRunning\n";
}
