// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef CKALLOC_H
#define CKALLOC_H

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "bri.h"

//#define CHECK_MEMORY

//#define WITH_PALLOC



#ifdef WITH_PALLOC

inline
void*
ckalloc(size_t amount) {
  return(palloc(amount));
}


#define ckfree(P)

#else

inline
void *
ckalloc(size_t amount) {

#ifdef __APPLE__
  if (amount == 0)
    amount = 16;
#else
  if (amount == 0)
    amount = 8;
#endif

  void *p = malloc(amount);

  if (p == NULL) {
    fprintf(stderr, "Can't allocate "u64bitFMT" bytes.\n", (u64bit)amount);
    kill(getpid(), SIGKILL);
  }

#ifdef CHECK_MEMORY
  fprintf(stderr, "ckalloc "u64bitHEX" "u64bitFMT"\n", (u64bit)p, (u64bit)amount);
#endif

  return(p);
}



#ifdef CHECK_MEMORY
#define ckfree(P) { fprintf(stderr, "ckfree "u64bitHEX" at %s,%d\n", (u64bit)(P), __FILE__, __LINE__); free(P); }
#else
#define ckfree(P) free(P)
#endif  //  CHECK_MEMORY

#endif  //  WITH_PALLOC


/* strsave -------------------------- save string s somewhere; return address */
inline
char *strsave(const char *s)
{
        char *p = (char *)ckalloc(strlen(s)+1);       /* +1 to hold '\0' */
        return strcpy(p, s);
}

#endif  //  CKALLOC_H
