/**************************************************************************
 * This file is part of ESTmapper.
 * Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
 * Authors: Brian Walenz and Liliana Florea
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received (LICENSE.txt) a copy of the GNU General Public 
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/


#include "sim4polish.h"

void
s4p_printPolish(FILE *O, sim4polish *o, u32bit flags) {
  int          i;
  char const  *mOri;
  char const  *sOri;
  sim4polish  *p = o;

  if (o == 0L)
    return;

  //  If there are flags, modify the polish before printing.
  //
  if (flags & (S4P_PRINTPOLISH_NORMALIZED |
               S4P_PRINTPOLISH_NOALIGNS |
               S4P_PRINTPOLISH_NODEFS)) {

    //  If the NOTVALUABLE flag is given, we modify the input polish.  Otherwise,
    //  we should copy the input polish to a temporary, and modify that.
    //
    if (!(flags & S4P_PRINTPOLISH_NOTVALUABLE))
      p = s4p_copyPolish(o);

    //  Normalized?
    //
    if (flags & S4P_PRINTPOLISH_NORMALIZED) {
      for (i=0; i<p->numExons; i++) {
        p->exons[i].genFrom += p->genLo;
        p->exons[i].genTo   += p->genLo;
      }
      p->genLo = 0;
      p->genHi = 0;
    }

    //  Remove alignments?
    //
    if (flags & S4P_PRINTPOLISH_NOALIGNS)
      s4p_removeAlignments(p);

    //  Remove deflines?
    //
    if (flags & S4P_PRINTPOLISH_NODEFS)
      s4p_removeDeflines(p);
  }

  fprintf(O, "sim4begin\n");

  switch (p->matchOrientation) {
    case SIM4_MATCH_FORWARD:
      mOri = "forward";
      break;
    case SIM4_MATCH_COMPLEMENT:
      mOri = "complement";
      break;
    default:
      fprintf(stderr, "sim4reader: Unknown matchOrientation '"u32bitFMT"' in printPolish()\n", p->matchOrientation);
      mOri = "UNKNOWN";
      break;
  }

  switch (p->strandOrientation) {
    case SIM4_STRAND_POSITIVE:
      sOri = "forward";
      break;
    case SIM4_STRAND_NEGATIVE:
      sOri = "reverse";
      break;
    case SIM4_STRAND_UNKNOWN:
      sOri = "unknown";
      break;
    case SIM4_STRAND_INTRACTABLE:
      sOri = "intractable";
      break;
    case SIM4_STRAND_FAILED:
      sOri = "aborted";
      break;
    default:
      fprintf(stderr, "sim4reader: Unknown strandOrientation '"u32bitFMT"' in printPolish()\n", p->matchOrientation);
      sOri = "UNKNOWN";
      break;
  }

  fprintf(O, ""u32bitFMT"["u32bitFMT"-"u32bitFMT"-"u32bitFMT"] "u32bitFMT"["u32bitFMT"-"u32bitFMT"] <"u32bitFMT"-"u32bitFMT"-"u32bitFMT"-%s-%s>\n",
          p->estID, p->estLen, p->estPolyA, p->estPolyT,
          p->genID, p->genLo, p->genHi,
          p->numMatches, p->numMatchesN, p->percentIdentity, mOri, sOri);

  if (p->comment)
    fprintf(O, "comment=%s\n", p->comment);
  if (p->estDefLine)
    fprintf(O, "edef=%s\n", p->estDefLine);
  if (p->genDefLine)
    fprintf(O, "ddef=%s\n", p->genDefLine);
  for (i=0; i<p->numExons; i++) {
    fprintf(O, ""u32bitFMT"-"u32bitFMT" ("u32bitFMT"-"u32bitFMT") <"u32bitFMT"-"u32bitFMT"-"u32bitFMT">",
            p->exons[i].estFrom, p->exons[i].estTo,
            p->exons[i].genFrom, p->exons[i].genTo,
            p->exons[i].numMatches, p->exons[i].numMatchesN, p->exons[i].percentIdentity);

    switch (p->exons[i].intronOrientation) {
      case SIM4_INTRON_POSITIVE:
        fprintf(O, " ->\n");
        break;
      case SIM4_INTRON_NEGATIVE:
        fprintf(O, " <-\n");
        break;
      case SIM4_INTRON_AMBIGUOUS:
        fprintf(O, " --\n");
        break;
      case SIM4_INTRON_GAP:
        fprintf(O, " ==\n");
        break;
      case SIM4_INTRON_NONE:
        fprintf(O, "\n");
        break;
      default:
        fprintf(O, "\n");
        break;
    }
  }
  for (i=0; i<p->numExons; i++) {
    if (p->exons[i].estAlignment)
      fprintf(O, "%s\n", p->exons[i].estAlignment);
    if (p->exons[i].genAlignment)
      fprintf(O, "%s\n", p->exons[i].genAlignment);
  }
  fprintf(O, "sim4end\n");

  if (p != o)
    s4p_destroyPolish(p);
}
