// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include "posix.H"
#include <stdio.h>
#include <stdlib.h>
#include "snapper2.H"
#include "bri++.H"

encodedQuery::encodedQuery(char const           *seq,
                           u32bit                seqLen,
                           u32bit                k,
                           bool                  rc) {

  _mersTotal = 0;
  _mersAvail = 0;
  _mers      = 0L;
  _skip      = 0L;

  if (k > seqLen)
    return;

  _mersTotal = seqLen - k + 1;
  _mersAvail = 0;
  _mers      = new u64bit [_mersTotal];
  _skip      = new u32bit [_mersTotal];

  u64bit   substring      = u64bitZERO;
  u64bit   mermask        = u64bitMASK(2 * k);
  s32bit   timeUntilValid = k;

  if (rc) {
    for (u32bit i=0; i<seqLen; ) {
      substring <<= 2;
      substring  &= mermask;

      if (validSymbol[seq[seqLen - 1 - i]]) {
        substring |= compressSymbol[ complementSymbol[ seq[seqLen - 1 - i] ]];
        timeUntilValid--;
      } else {
        timeUntilValid = k;
      }

      i++;

      if (i >= k) {
        _mers[i-k]  = substring;
        _skip[i-k]  = timeUntilValid > 0;
        _mersAvail += 1 - _skip[i-k];
      }
    }
  } else {
    for (u32bit i=0; i<seqLen; ) {
      substring <<= 2;
      substring  &= mermask;

      if (validSymbol[seq[i]]) {
        substring |= compressSymbol[ seq[i] ];
        timeUntilValid--;
      } else {
        timeUntilValid = k;
      }

      i++;

      if (i >= k) {
        _mers[i-k]  = substring;
        _skip[i-k]  = timeUntilValid > 0;
        _mersAvail += 1 - _skip[i-k];
      }
    }
  }
}

encodedQuery::~encodedQuery() {
  delete [] _mers;
  delete [] _skip;
}
