/**************************************************************************
 * This file is part of ESTmapper.
 * Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
 * Authors: Brian Walenz and Liliana Florea
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received (LICENSE.txt) a copy of the GNU General Public 
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
 
/*
 *  This is a short example of memory debugging on AIX.  It uses
 *  an xlC supplied function (it might work with gcc, but I've not
 *  tested) that will report all memory segments allocated since
 *  the last time it was called.
 *
 *    _dump_allocated_delta(fileno(stdout));
 *
 *  This is described somewhere in Chapter 7 of the "C for AIX User's
 *  Guide".
 *
 *  Compile with the -qheapdebug switch:
 *
 *    cc -qheapdebug -o memdebug memdebug.c
 *
 *  Output looks like
 *
 *  1546-515 -----------------------------------------------------------------
 *  1546-526                  START OF DELTA DUMP OF ALLOCATED MEMORY BLOCKS
 *  1546-515 -----------------------------------------------------------------
 *  1546-518 Address: 0x000000011022B5A0      Size: 0x00000000000003E8 (1000)
 *  1546-519 This object was (re)allocated at line 23 in memdebug.c.
 *                 _debug_umalloc + 6C
 *            _debug_umalloc_init + 44
 *                  _debug_malloc + 48
 *                           main + 34
 *  1546-520 Memory contents: 43                                     [C               ]
 *  1546-515 -----------------------------------------------------------------
 *  1546-517                  END OF DUMP OF ALLOCATED MEMORY BLOCKS
 *  1546-515 -----------------------------------------------------------------
 *
 */

int
main(int argc, char **argv) {
  char   *fixed;
  char   *leak;
  int     i;

  fixed = (char *)malloc(sizeof(char) * 1000);

  _dump_allocated(0);

  for (i=0; i<16; i++) {
    if ((i % 2) == 0)
      leak = (char *)malloc(sizeof(char) * i);

    if ((i % 4) == 0)
      free(leak);

    fprintf(stderr, "\n*** Begin of dump for loop iteration %d ***\n", i);
    _dump_allocated_delta(1);

    _heap_check();
  }
}
