// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#include <stdio.h>
#include "bri++.H"

//  A quick test to filter sequences by the percentage of allowed
//  letters.

const char *usage =
"usage: %s [-ratio ratio] [sequence options] [-pass pass.fasta] [-fail fail.fasta] input.fasta\n"
"\n"
"       sequence options:\n"
"         -acgt:     valid sequence is [acgtACGT]\n"
"         -ACGT:     valid sequence is [ACGT]\n"
"         -dna:      valid sequence is []\n"
"         -DNA:      valid sequence is []\n"
"         -protein:  valid sequence is []\n"
"         -PROTEIN:  valid sequence is []\n";

void
setValid(unsigned char *val, char const *set) {
  while (*set)
    val[*(set++)] = 1;
}

int
main(int argc, char **argv) {
  FastABuffer           B;
  FastA                *I = 0L;
  double                percent = 0.5;
  FILE                 *passFile = 0L;
  FILE                 *failFile = 0L;
  const unsigned char  *seq = 0L;
  u32bit                len = 0;
  unsigned char         val[256] = { 0 };
  u32bit                sco = 0;
  u32bit                pass = 0;
  u32bit                fail = 0;

  if (argc < 2) {
    fprintf(stderr, usage, argv[0]);
    exit(1);
  }

  int arg = 1;
  while (arg < argc) {
    if        (strncmp(argv[arg], "-ratio", 2) == 0) {
      percent = atof(argv[++arg]);
    } else if (strncmp(argv[arg], "-pass", 3) == 0) {
      passFile = fopen(argv[++arg], "w");
    } else if (strncmp(argv[arg], "-fail", 2) == 0) {
      failFile = fopen(argv[++arg], "w");
    } else if (strncmp(argv[arg], "-acgt", 2) == 0) {
      setValid(val, "acgtACGT");
    } else if (strncmp(argv[arg], "-ACGT", 2) == 0) {
      setValid(val, "ACGT");
    } else if (strncmp(argv[arg], "-dna", 2) == 0) {
      fprintf(stderr, "Someone needs to write this!\n");
    } else if (strncmp(argv[arg], "-DNA", 2) == 0) {
      fprintf(stderr, "Someone needs to write this!\n");
    } else if (strncmp(argv[arg], "-protein", 3) == 0) {
      fprintf(stderr, "Someone needs to write this!\n");
    } else if (strncmp(argv[arg], "-PROTEIN", 2) == 0) {
      fprintf(stderr, "Someone needs to write this!\n");
    } else {
      I = new FastA(argv[arg]);
    }
    arg++;
  }

  for (I->first(B); !I->eof(); I->next(B)) {
    seq = B.sequence();
    len = B.sequenceLength();
    sco = 0;

    for (u32bit i=0; i<len; i++)
      sco += val[seq[i]];

    if (sco >= percent * len) {
      pass++;
      if (passFile)
        fprintf(passFile, ">%s\n%s\n", B.header()+1, seq);
    } else {
      fail++;
      if (failFile)
        fprintf(failFile, ">%s\n%s\n", B.header()+1, seq);
    }
  }

  delete I;

#ifdef TRUE64BIT
  fprintf(stderr, "%u pass, %u fail\n", pass, fail);
#else
  fprintf(stderr, "%lu pass, %lu fail\n", pass, fail);
#endif
}

