// This file is part of ESTmapper.
// Copyright (c) 2001, 2002, 2003, 2004 Applera Corporation
// Authors: Brian Walenz and Liliana Florea
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received (LICENSE.txt) a copy of the GNU General Public 
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef FASTA_CACHE_H
#define FASTA_CACHE_H

#include "bri++.H"

//  Keeps a cache of the last N sequences accessed.  If N==0, then the
//  cache is unlimited in size.

class FastACache {
public:
  FastACache(char *filename, u32bit cachesize=32, bool loadall=false, bool report=false);
  ~FastACache();

  //  Returns sequence iid.  If the sequence is not in the cache,
  //  the sequence is loaded.
  //
  FastASequenceInCore    *getSequence(u32bit iid);

  //  Returns a pointer to the fasta wrapper we are using.
  //
  FastAWrapper           *fasta(void) {
    return(_fastawrapper);
  };

private:
  FastAWrapper          *_fastawrapper;

  bool                   _allSequencesLoaded;
  bool                   _reportLoading;

  //  cacheSize:  The size of the cache.  If non-zero, this is the size
  //  of _cacheMap and _seqs.
  //
  u32bit                *_cacheMap;
  u32bit                 _cacheSize;
  u32bit                 _cacheNext;
  FastASequenceInCore  **_cache;
};

#endif  //  FASTA_CACHE_H
